@@ -41,23 +41,23 @@
 #include "npapi.h"
 #include "npupp.h"
 
-#include <nsCOMPtr.h>
 #include <nsIDOMWindow.h>
 #include <nsIURI.h>
 #include <nsEmbedString.h>
 #include <nsIInterfaceRequestorUtils.h>
 #include <docshell/nsIWebNavigation.h>
 
-#define DEBUG(x) printf(x "\n")
-//#define DEBUG(x)
+/* define TOTEM_DEBUG for more debug spew */
+#include "debug.h"
 
 static NPNetscapeFuncs mozilla_functions;
+static char *mime_list = NULL;
 
 static void
 cb_update_name (DBusGProxy *proxy, char *svc, char *old_owner,
 		char *new_owner, TotemPlugin *plugin)
 {
-	g_print ("Received notification for %s\n", svc);
+	D("Received notification for %s\n", svc);
 	if (strcmp (svc, plugin->wait_for_svc) == 0) {
 		plugin->got_svc = TRUE;
 	}
@@ -115,6 +115,7 @@ totem_plugin_fork (TotemPlugin *plugin)
 	argv[argc++] = g_strdup ("fd://0");
 	argv[argc] = NULL;
 
+#ifdef TOTEM_DEBUG
 	{
 		int i;
 		g_print ("Launching: ");
@@ -123,13 +124,14 @@ totem_plugin_fork (TotemPlugin *plugin)
 		}
 		g_print ("\n");
 	}
+#endif
 
 	if (g_spawn_async_with_pipes (NULL, argv, NULL,
 				G_SPAWN_DO_NOT_REAP_CHILD,
 				NULL, NULL, &plugin->player_pid,
 				&plugin->send_fd, NULL, NULL, &err) == FALSE)
 	{
-		DEBUG("Spawn failed");
+		D("Spawn failed");
 
 		if (err)
 		{
@@ -148,7 +150,7 @@ totem_plugin_fork (TotemPlugin *plugin)
 	plugin->wait_for_svc =
 		g_strdup_printf ("org.totem_%d.MozillaPluginService",
 				 plugin->player_pid);
-	g_print ("waiting for signal %s\n", plugin->wait_for_svc);
+	D("waiting for signal %s", plugin->wait_for_svc);
 	dbus_g_proxy_add_signal (plugin->proxy, "NameOwnerChanged",
 				 G_TYPE_STRING, G_TYPE_STRING,
 				 G_TYPE_STRING, G_TYPE_INVALID);
@@ -166,7 +168,7 @@ totem_plugin_fork (TotemPlugin *plugin)
 	if (!iface->tm) {
 		/* we were destroyed in one of the iterations of the
 		 * mainloop, get out ASAP */
-		fprintf (stderr, "We no longer exist\n");
+		D("We no longer exist");
 		NS_RELEASE (iface);
 		return FALSE;
 	}
@@ -192,7 +194,7 @@ totem_plugin_fork (TotemPlugin *plugin)
 					   "/TotemEmbedded",
 					   "org.totem.MozillaPluginInterface");
 	g_free (plugin->wait_for_svc);
-	g_print ("Done forking, new proxy=%p\n", plugin->proxy);
+	D("Done forking, new proxy=%p", plugin->proxy);
 
 	return TRUE;
 }
@@ -220,11 +222,16 @@ static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
 	GError *e = NULL;
 	int i;
 
-	DEBUG("totem_plugin_new_instance");
+	D("totem_plugin_new_instance");
 
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
 
+	/* Make sure the plugin stays resident to avoid crashers when 
+	 * reloading the GObject types */
+	mozilla_functions.setvalue (instance,
+			NPPVpluginKeepLibraryInMemory, (void *)TRUE);
+
 	instance->pdata = mozilla_functions.memalloc(sizeof(TotemPlugin));
 	plugin = (TotemPlugin *) instance->pdata;
 
@@ -263,14 +270,20 @@ static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
 	plugin->send_fd = -1;
 
 	/* to resolve relative URLs */
-	nsCOMPtr<nsIDOMWindow> domWin;
-	mozilla_functions.getvalue (instance, NPNVDOMWindow, 
-				    NS_STATIC_CAST (nsIDOMWindow **, getter_AddRefs (domWin)));
-	nsCOMPtr<nsIWebNavigation> webNav (do_GetInterface (domWin));
+	nsIDOMWindow *domWin = nsnull;
+	mozilla_functions.getvalue (instance, NPNVDOMWindow,
+				    NS_REINTERPRET_CAST (void**, &domWin));
+	nsIWebNavigation *webNav = nsnull;
+	if (domWin) {
+		domWin->QueryInterface (NS_GET_IID (nsIWebNavigation),
+					NS_REINTERPRET_CAST (void**, &webNav));
+		NS_RELEASE (domWin);
+	}
 
-	nsCOMPtr<nsIURI> docURI;
+	nsIURI *docURI = nsnull;
 	if (webNav) {
-		webNav->GetCurrentURI (getter_AddRefs (docURI));
+		webNav->GetCurrentURI (&docURI);
+		NS_RELEASE (webNav);
 	}
 
 	for (i=0; i<argc; i++) {
@@ -312,6 +325,8 @@ static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
 		}
 	}
 
+	NS_IF_RELEASE (docURI);	
+
 	return NPERR_NO_ERROR;
 }
 
@@ -319,7 +334,7 @@ static NPError totem_plugin_destroy_instance (NPP instance, NPSavedData **save)
 {
 	TotemPlugin * plugin;
 
-	DEBUG("plugin_destroy");
+	D("plugin_destroy");
 
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
@@ -356,7 +371,7 @@ static NPError totem_plugin_set_window (NPP instance, NPWindow* window)
 {
 	TotemPlugin *plugin;
 
-	DEBUG("plugin_set_window");
+	D("plugin_set_window");
 
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
@@ -366,19 +381,19 @@ static NPError totem_plugin_set_window (NPP instance, NPWindow* window)
 		return NPERR_INVALID_INSTANCE_ERROR;
 
 	if (plugin->window) {
-		DEBUG ("existing window");
+		D ("existing window");
 		if (plugin->window == (Window) window->window) {
-			DEBUG("resize");
+			D("resize");
 			/* Resize event */
 			/* Not currently handled */
 		} else {
-			DEBUG("change");
+			D("change");
 			printf ("ack.  window changed!\n");
 		}
 	} else {
 		gchar *msg;
 
-		DEBUG("about to fork");
+		D("about to fork");
 
 		plugin->window = (Window) window->window;
 		if (!totem_plugin_fork (plugin))
@@ -388,7 +403,7 @@ static NPError totem_plugin_set_window (NPP instance, NPWindow* window)
 			fcntl(plugin->send_fd, F_SETFL, O_NONBLOCK);
 	}
 
-	DEBUG("leaving plugin_set_window");
+	D("leaving plugin_set_window");
 
 	return NPERR_NO_ERROR;
 }
@@ -396,7 +411,7 @@ static NPError totem_plugin_set_window (NPP instance, NPWindow* window)
 static NPError totem_plugin_new_stream (NPP instance, NPMIMEType type,
 		NPStream* stream_ptr, NPBool seekable, uint16* stype)
 {
-	DEBUG("plugin_new_stream");
+	D("plugin_new_stream");
 
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
@@ -409,10 +424,10 @@ static NPError totem_plugin_destroy_stream (NPP instance, NPStream* stream,
 {
 	TotemPlugin *plugin;
 
-	DEBUG("plugin_destroy_stream");
+	D("plugin_destroy_stream");
 
 	if (instance == NULL) {
-		DEBUG("totem_plugin_destroy_stream instance is NULL");
+		D("totem_plugin_destroy_stream instance is NULL");
 		return NPERR_NO_ERROR;
 	}
 
@@ -428,10 +443,10 @@ static int32 totem_plugin_write_ready (NPP instance, NPStream *stream)
 {
 	TotemPlugin *plugin;
 
-	DEBUG("plugin_write_ready");
+	D("plugin_write_ready");
 
 	if (instance == NULL) {
-		DEBUG("plugin_write_ready instance is NULL");
+		D("plugin_write_ready instance is NULL");
 		return 0;
 	}
 
@@ -449,7 +464,7 @@ static int32 totem_plugin_write (NPP instance, NPStream *stream, int32 offset,
 	TotemPlugin *plugin;
 	int ret;
 
-	DEBUG("plugin_write");
+	D("plugin_write");
 
 	if (instance == NULL)
 		return -1;
@@ -479,7 +494,7 @@ static void totem_plugin_stream_as_file (NPP instance, NPStream *stream,
 {
 	TotemPlugin *plugin;
 
-	DEBUG("plugin_stream_as_file");
+	D("plugin_stream_as_file");
 
 	if (instance == NULL)
 		return;
@@ -495,7 +510,7 @@ static void
 totem_plugin_url_notify (NPP instance, const char* url,
 		NPReason reason, void* notifyData)
 {
-	DEBUG("plugin_url_notify");
+	D("plugin_url_notify");
 }
 
 static char *
@@ -530,7 +545,7 @@ totem_plugin_get_value (NPP instance, NPPVariable variable,
 		if (ptr) {
 			*ptr = sIID;
 			* (nsIID **) value = ptr;
-			g_print ("Returning that we support iface\n");
+			D("Returning that we support iface");
 		} else {
 			err = NPERR_OUT_OF_MEMORY_ERROR;
 		}
@@ -546,12 +561,12 @@ totem_plugin_get_value (NPP instance, NPPVariable variable,
 			plugin->iface->QueryInterface (NS_GET_IID (nsISupports),
 						       (void **) value);
 //			* (nsISupports **) value = static_cast<totemMozillaScript *>(plugin->iface);
-			g_print ("Returning instance %p\n", plugin->iface);
+			D("Returning instance %p", plugin->iface);
 		}
 		break;
 	}
 	default:
-		g_message ("unhandled variable %d", variable);
+		D("unhandled variable %d", variable);
 		err = NPERR_INVALID_PARAM;
 		break;
 	}
@@ -563,7 +578,7 @@ static NPError
 totem_plugin_set_value (NPP instance, NPNVariable variable,
 		                        void *value)
 {
-	DEBUG("plugin_set_value");
+	D("plugin_set_value");
 
 	return NPERR_NO_ERROR;
 }
@@ -574,24 +589,25 @@ NP_GetValue(void *future, NPPVariable variable, void *value)
 	return totem_plugin_get_value (NULL, variable, value);
 }
 
-#define NUM_MIME_TYPES 6
 static struct {
 	const char *mime_type;
 	const char *extensions;
 	const char *mime_alias;
 } mime_types[] = {
-	{ "video/quicktime", "mov" },
+	{ "video/quicktime", "mov", NULL },
 	{ "application/x-mplayer2", "avi, wma, wmv", "video/x-msvideo" },
-	{ "video/mpeg", "mpg, mpeg, mpe" },
+	{ "video/mpeg", "mpg, mpeg, mpe", NULL },
 	{ "video/x-ms-asf-plugin", "asf, wmv", "video/x-ms-asf" },
 	{ "video/x-ms-wmv", "wmv", "video/x-ms-asf" },
-	{ "application/ogg", "ogg" }
+	{ "application/ogg", "ogg", NULL },
+	{ "application/x-google-vlc-plugin", "", "Google VLC plugin" },
+	{ "application/x-vlc-plugin", "", "VideoLAN plugin" }
 };
+#define NUM_MIME_TYPES G_N_ELEMENTS(mime_types)
 
 char *NP_GetMIMEDescription(void)
 {
 	GString *list;
-	static char *mime_list = NULL;
 	guint i;
 
 	if (mime_list != NULL)
@@ -604,10 +620,14 @@ char *NP_GetMIMEDescription(void)
 		char *item;
 
 		desc = gnome_vfs_mime_get_description (mime_types[i].mime_type);
-		if (desc == NULL) {
+		if (desc == NULL && mime_types[i].mime_alias != NULL) {
 			desc = gnome_vfs_mime_get_description
 				(mime_types[i].mime_alias);
 		}
+		if (desc == NULL) {
+			desc = mime_types[i].mime_alias;
+		}
+
 		item = g_strdup_printf ("%s:%s:%s;", mime_types[i].mime_type,
 				mime_types[i].extensions, desc);
 		list = g_string_append (list, item);
@@ -721,6 +741,9 @@ NPError NP_Initialize (NPNetscapeFuncs * moz_funcs,
 
 NPError NP_Shutdown(void)
 {
+	g_free (mime_list);
+	mime_list = NULL;
+
 	return NPERR_NO_ERROR;
 }
 
