@@ -33,6 +33,11 @@
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <string.h>
 
+#ifdef HAVE_LIBMUSICBRAINZ
+#include <musicbrainz/mb_c.h>
+#endif
+
+#include "totem-uri.h"
 #include "totem-interface.h"
 #include "totem-pl-parser.h"
 #include "debug.h"
@@ -84,11 +89,18 @@ struct TotemPlaylistPrivate
 	GConfClient *gc;
 
 	int x, y;
+
+	/* CD index caching */
+	char *cdindex;
+
+	gboolean disable_save_to_disk;
 };
 
 /* Signals */
 enum {
 	CHANGED,
+	ITEM_ACTIVATED,
+	ACTIVE_NAME_CHANGED,
 	CURRENT_REMOVED,
 	REPEAT_TOGGLED,
 	SHUFFLE_TOGGLED,
@@ -100,6 +112,9 @@ enum {
 	FILENAME_COL,
 	URI_COL,
 	TITLE_CUSTOM_COL,
+	CACHE_TITLE_COL,
+	CACHE_ARTIST_COL,
+	CACHE_ALBUM_COL,
 	NUM_COLS
 };
 
@@ -119,7 +134,7 @@ static void init_treeview (GtkWidget *treeview, TotemPlaylist *playlist);
 
 #define totem_playlist_unset_playing(x) totem_playlist_set_playing(x, FALSE)
 
-G_DEFINE_TYPE(TotemPlaylist, totem_playlist, GTK_TYPE_DIALOG)
+G_DEFINE_TYPE(TotemPlaylist, totem_playlist, GTK_TYPE_VBOX)
 
 /* Helper functions */
 static gboolean
@@ -163,13 +178,19 @@ totem_playlist_gtk_tree_path_equals (GtkTreePath *path1, GtkTreePath *path2)
 	return retval;
 }
 
+static GtkWindow *
+totem_playlist_get_toplevel (TotemPlaylist *playlist)
+{
+	return GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (playlist)));
+}
+
 static void
 totem_playlist_error (char *title, char *reason, TotemPlaylist *playlist)
 {
 	GtkWidget *error_dialog;
 
 	error_dialog =
-		gtk_message_dialog_new (GTK_WINDOW (playlist),
+		gtk_message_dialog_new (totem_playlist_get_toplevel (playlist),
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_OK,
@@ -236,6 +257,17 @@ totem_playlist_is_media (const char *mrl)
 	return FALSE;
 }
 
+static void
+totem_playlist_update_save_button (TotemPlaylist *playlist)
+{
+	GtkWidget *button;
+	gboolean state;
+
+	button = glade_xml_get_widget (playlist->_priv->xml, "save_button");
+	state = (!playlist->_priv->disable_save_to_disk) && (PL_LEN != 0);
+	gtk_widget_set_sensitive (button, state);
+}
+
 static char*
 totem_playlist_create_full_path (const char *path)
 {
@@ -336,7 +368,7 @@ drop_cb (GtkWidget     *widget,
 {
 	GList *list, *p, *file_list;
 
-	list = gnome_vfs_uri_list_parse (data->data);
+	list = gnome_vfs_uri_list_parse ((char *)data->data);
 
 	if (list == NULL) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
@@ -365,37 +397,28 @@ drop_cb (GtkWidget     *widget,
 
 	for (p = file_list; p != NULL; p = p->next)
 	{
-		char *filename;
+		char *filename, *title;
 
 		if (p->data == NULL)
 			continue;
 
-		filename = gnome_vfs_get_local_path_from_uri (p->data);
-		if (filename == NULL)
-			filename = g_strdup (p->data);
+		filename = totem_create_full_path (p->data);
+		title = NULL;
 
-		if (filename != NULL &&
-				(g_file_test (filename, G_FILE_TEST_IS_REGULAR
-					| G_FILE_TEST_EXISTS)
-				 || strstr (filename, "://") != NULL))
+		if (info == 1)
 		{
-			char *title = NULL;
-
-			/* Super _NETSCAPE_URL trick */
-			if (info == 1)
-			{
-				g_free (p->data);
-				p = p->next;
-				if (p != NULL) {
-					if (g_str_has_prefix (p->data, "file:") != FALSE)
-						title = (char *) p->data + 5;
-					else
-						title = p->data;
-				}
+			g_free (p->data);
+			p = p->next;
+			if (p != NULL) {
+				if (g_str_has_prefix (p->data, "file:") != FALSE)
+					title = (char *)p->data + 5;
+				else
+					title = p->data;
 			}
-
-			totem_playlist_add_mrl (playlist, p->data, title);
 		}
+
+		totem_playlist_add_mrl (playlist, p->data, title);
+
 		g_free (filename);
 		g_free (p->data);
 	}
@@ -487,6 +510,7 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 		GtkTreeIter iter;
 		char *index;
 		GdkPixbuf *pixbuf;
+		GtkTreePath *path;
 
 		index = g_strdup_printf ("%d", i);
 		if (gtk_tree_model_get_iter_from_string
@@ -505,11 +529,19 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 			continue;
 
 		gdk_pixbuf_unref (pixbuf);
-		gtk_tree_path_free (playlist->_priv->current);
-		playlist->_priv->current = gtk_tree_path_new_from_indices (i, -1);
-		g_signal_emit (G_OBJECT (playlist),
-				totem_playlist_table_signals[CHANGED],
-				0, NULL);
+
+		/* Only emit the changed signal if we changed the ->current */
+		path = gtk_tree_path_new_from_indices (i, -1);
+		if (gtk_tree_path_compare (path, playlist->_priv->current) == 0)
+		{
+			gtk_tree_path_free (path);
+		} else {
+			gtk_tree_path_free (playlist->_priv->current);
+			playlist->_priv->current = path;
+			g_signal_emit (G_OBJECT (playlist),
+					totem_playlist_table_signals[CHANGED],
+					0, NULL);
+		}
 
 		break;
 	}
@@ -626,58 +658,28 @@ update_current_from_playlist (TotemPlaylist *playlist)
 static void
 totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
-	GtkWidget *fs;
-	int response;
+	GSList *filenames, *l;
+	char *new_path;
 
-	fs = gtk_file_chooser_dialog_new (_("Select Movies or Playlists"),
-			GTK_WINDOW (playlist), GTK_FILE_CHOOSER_ACTION_OPEN,
-			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT,
-			NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
-	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fs), TRUE);
-	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
+	filenames = totem_add_files (totem_playlist_get_toplevel (playlist),
+			playlist->_priv->path, &new_path);
+	if (filenames == NULL)
+		return;
 
-	if (playlist->_priv->path != NULL)
-	{
-		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (fs),
-				playlist->_priv->path);
+	if (new_path != NULL) {
+		g_free (playlist->_priv->path);
+		playlist->_priv->path = new_path;
 	}
-	response = gtk_dialog_run (GTK_DIALOG (fs));
-	gtk_widget_hide (fs);
-	while (gtk_events_pending())
-		gtk_main_iteration();
 
-	if (response == GTK_RESPONSE_ACCEPT)
-	{
-		GSList *filenames, *l;
+	for (l = filenames; l != NULL; l = l->next) {
 		char *mrl;
 
-		filenames = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (fs));
-		if (filenames == NULL)
-		{
-			gtk_widget_destroy (fs);
-			return;
-		}
-
-		mrl = filenames->data;
-		if (mrl != NULL)
-		{
-			g_free (playlist->_priv->path);
-			playlist->_priv->path = g_path_get_dirname (mrl);
-		}
-
-		for (l = filenames; l != NULL; l = l->next)
-		{
-			mrl = l->data;
-			totem_playlist_add_mrl (playlist, mrl, NULL);
-			g_free (mrl);
-		}
-
-		g_slist_free (filenames);
+		mrl = l->data;
+		totem_playlist_add_mrl (playlist, mrl, NULL);
+		g_free (mrl);
 	}
 
-	gtk_widget_destroy (fs);
+	g_slist_free (filenames);
 }
 
 static void
@@ -798,6 +800,7 @@ totem_playlist_remove_files (GtkWidget *widget, TotemPlaylist *playlist)
 				totem_playlist_table_signals[CHANGED], 0,
 				NULL);
 	}
+	totem_playlist_update_save_button (playlist);
 }
 
 static void
@@ -806,8 +809,9 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 	GtkWidget *fs;
 	int response;
 
-	fs = gtk_file_chooser_dialog_new (_("Save playlist"),
-			GTK_WINDOW (playlist), GTK_FILE_CHOOSER_ACTION_SAVE,
+	fs = gtk_file_chooser_dialog_new (_("Save Playlist"),
+			totem_playlist_get_toplevel (playlist),
+			GTK_FILE_CHOOSER_ACTION_SAVE,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 			NULL);
@@ -847,7 +851,7 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 			GtkWidget *dialog;
 
 			dialog = gtk_message_dialog_new
-				(GTK_WINDOW (playlist),
+				(totem_playlist_get_toplevel (playlist),
 				 GTK_DIALOG_MODAL,
 				 GTK_MESSAGE_QUESTION,
 				 GTK_BUTTONS_NONE,
@@ -932,7 +936,9 @@ totem_playlist_move_files (TotemPlaylist *playlist, gboolean direction_up)
 	g_list_foreach (paths, (GFunc) gtk_tree_path_free, NULL);
 	g_list_free (paths);
 
-	refs = g_list_reverse (refs);
+	/* Otherwise we reverse the items when moving down */
+	if (direction_up != FALSE)
+		refs = g_list_reverse (refs);
 
 	if (direction_up == FALSE)
 		pos = pos + 2;
@@ -1052,7 +1058,6 @@ init_columns (GtkTreeView *treeview)
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	gtk_tree_view_column_set_attributes (column, renderer,
 			"pixbuf", PIX_COL, NULL);
-	gtk_tree_view_column_set_title (column, _("Filename"));
 	gtk_tree_view_append_column (treeview, column);
 
 	/* Labels */
@@ -1070,7 +1075,7 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 			(arg1, playlist->_priv->current) != FALSE)
 	{
 		g_signal_emit (G_OBJECT (playlist),
-				totem_playlist_table_signals[CHANGED], 0,
+				totem_playlist_table_signals[ITEM_ACTIVATED], 0,
 				NULL);
 		return;
 	}
@@ -1118,11 +1123,15 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 				GDK_TYPE_PIXBUF,
 				G_TYPE_STRING,
 				G_TYPE_STRING,
-				G_TYPE_BOOLEAN));
+				G_TYPE_BOOLEAN,
+				G_TYPE_STRING,
+				G_TYPE_STRING,
+				G_TYPE_STRING));
 
 	/* the treeview */
 	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), model);
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
 	g_object_unref (G_OBJECT (model));
 
 	init_columns (GTK_TREE_VIEW (treeview));
@@ -1156,38 +1165,14 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 	gtk_widget_show (treeview);
 }
 
-static void
-repeat_button_toggled (GtkToggleButton *togglebutton, TotemPlaylist *playlist)
-{
-	gboolean repeat;
-
-	repeat = gtk_toggle_button_get_active (togglebutton);
-	gconf_client_set_bool (playlist->_priv->gc, GCONF_PREFIX"/repeat",
-			repeat, NULL);
-	playlist->_priv->repeat = repeat;
-
-	g_signal_emit (G_OBJECT (playlist),
-			totem_playlist_table_signals[REPEAT_TOGGLED], 0,
-			repeat, NULL);
-}
-
 static void
 update_repeat_cb (GConfClient *client, guint cnxn_id,
 		GConfEntry *entry, TotemPlaylist *playlist)
 {
-	GtkWidget *button;
 	gboolean repeat;
 
-	repeat = gconf_client_get_bool (client,
-			GCONF_PREFIX"/repeat", NULL);
-	button = glade_xml_get_widget (playlist->_priv->xml, "repeat_button");
-	g_signal_handlers_disconnect_by_func (G_OBJECT (button),
-			repeat_button_toggled, playlist);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), repeat);
+	repeat = gconf_value_get_bool (entry->value);
 	playlist->_priv->repeat = repeat;
-	g_signal_connect (G_OBJECT (button), "toggled",
-			G_CALLBACK (repeat_button_toggled),
-			(gpointer) playlist);
 
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
@@ -1270,39 +1255,15 @@ ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle)
 	g_array_free (array, TRUE);
 }
 
-static void
-shuffle_button_toggled (GtkToggleButton *togglebutton, TotemPlaylist *playlist)
-{
-	gboolean shuffle;
-
-	shuffle = gtk_toggle_button_get_active (togglebutton);
-	gconf_client_set_bool (playlist->_priv->gc, GCONF_PREFIX"/shuffle",
-			shuffle, NULL);
-	playlist->_priv->shuffle = shuffle;
-	ensure_shuffled (playlist, shuffle);
-
-	g_signal_emit (G_OBJECT (playlist),
-			totem_playlist_table_signals[SHUFFLE_TOGGLED], 0,
-			shuffle, NULL);
-}
-
 static void
 update_shuffle_cb (GConfClient *client, guint cnxn_id,
 		GConfEntry *entry, TotemPlaylist *playlist)
 {
-	GtkWidget *button;
 	gboolean shuffle;
 
-	shuffle = gconf_client_get_bool (client,
-			GCONF_PREFIX"/shuffle", NULL);
-	button = glade_xml_get_widget (playlist->_priv->xml, "shuffle_button");
-	g_signal_handlers_disconnect_by_func (G_OBJECT (button),
-			shuffle_button_toggled, playlist);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), shuffle);
+	shuffle = gconf_value_get_bool (entry->value);
 	playlist->_priv->shuffle = shuffle;
-	g_signal_connect (G_OBJECT (button), "toggled",
-			G_CALLBACK (shuffle_button_toggled),
-			(gpointer) playlist);
+	ensure_shuffled (playlist, shuffle);
 
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
@@ -1316,43 +1277,21 @@ static void
 update_lockdown (GConfClient *client, guint cnxn_id,
 		GConfEntry *entry, TotemPlaylist *playlist)
 {
-	GtkWidget *button;
-	gboolean locked;
-
-	locked = gconf_client_get_bool (playlist->_priv->gc,
+	playlist->_priv->disable_save_to_disk = gconf_client_get_bool
+			(playlist->_priv->gc,
 			"/desktop/gnome/lockdown/disable_save_to_disk", NULL);
-	button = glade_xml_get_widget (playlist->_priv->xml, "save_button");
-	gtk_widget_set_sensitive (button, !locked);
+	totem_playlist_update_save_button (playlist);
 }
 
 static void
 init_config (TotemPlaylist *playlist)
 {
-	GtkWidget *button;
-	gboolean repeat, shuffle, locked;
-
 	playlist->_priv->gc = gconf_client_get_default ();
 
-	button = glade_xml_get_widget (playlist->_priv->xml, "repeat_button");
-	repeat = gconf_client_get_bool (playlist->_priv->gc,
-			GCONF_PREFIX"/repeat", NULL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), repeat);
-	g_signal_connect (G_OBJECT (button), "toggled",
-			G_CALLBACK (repeat_button_toggled),
-			(gpointer) playlist);
-
-	button = glade_xml_get_widget (playlist->_priv->xml, "shuffle_button");
-	shuffle = gconf_client_get_bool (playlist->_priv->gc,
-			GCONF_PREFIX"/shuffle", NULL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), shuffle);
-	g_signal_connect (G_OBJECT (button), "toggled",
-			G_CALLBACK (shuffle_button_toggled),
-			(gpointer) playlist);
-
-	locked = gconf_client_get_bool (playlist->_priv->gc,
+	playlist->_priv->disable_save_to_disk = gconf_client_get_bool
+	       		(playlist->_priv->gc,
 			"/desktop/gnome/lockdown/disable_save_to_disk", NULL);
-	button = glade_xml_get_widget (playlist->_priv->xml, "save_button");
-	gtk_widget_set_sensitive (button, !locked);
+	totem_playlist_update_save_button (playlist);
 
 	gconf_client_add_dir (playlist->_priv->gc, GCONF_PREFIX,
 			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
@@ -1370,8 +1309,10 @@ init_config (TotemPlaylist *playlist)
 			(GConfClientNotifyFunc) update_lockdown,
 			playlist, NULL, NULL);
 
-	playlist->_priv->repeat = repeat;
-	playlist->_priv->shuffle = shuffle;
+	playlist->_priv->repeat = gconf_client_get_bool (playlist->_priv->gc,
+			GCONF_PREFIX"/repeat", NULL);
+	playlist->_priv->shuffle = gconf_client_get_bool (playlist->_priv->gc,
+			GCONF_PREFIX"/shuffle", NULL);
 }
 
 static void
@@ -1396,8 +1337,6 @@ totem_playlist_init (TotemPlaylist *playlist)
 			"entry",
 			G_CALLBACK (totem_playlist_entry_parsed),
 			playlist);
-
-	gtk_container_set_border_width (GTK_CONTAINER (playlist), 5);
 }
 
 static void
@@ -1418,83 +1357,11 @@ totem_playlist_finalize (GObject *object)
 	}
 }
 
-static void
-totem_playlist_unrealize (GtkWidget *widget)
-{
-	TotemPlaylist *playlist = TOTEM_PLAYLIST (widget);
-	int x, y;
-
-	g_return_if_fail (widget != NULL);
-
-	if (GTK_WIDGET_MAPPED (widget) != FALSE)
-	{
-		gtk_window_get_position (GTK_WINDOW (widget), &x, &y);
-	} else {
-		x = playlist->_priv->x;
-		y = playlist->_priv->y;
-	}
-
-	gconf_client_set_int (playlist->_priv->gc, GCONF_PREFIX"/playlist_x",
-			x, NULL);
-	gconf_client_set_int (playlist->_priv->gc, GCONF_PREFIX"/playlist_y",
-			y, NULL);
-
-	GTK_WIDGET_UNSET_FLAGS (widget, GTK_REALIZED);
-
-	if (GTK_WIDGET_CLASS (parent_class)->unrealize != NULL) {
-		(* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
-	}
-}
-
-static void
-totem_playlist_unmap (GtkWidget *widget)
-{
-	TotemPlaylist *playlist = TOTEM_PLAYLIST (widget);
-	int x, y;
-
-	g_return_if_fail (widget != NULL);
-
-	gtk_window_get_position (GTK_WINDOW (widget), &x, &y);
-	playlist->_priv->x = x;
-	playlist->_priv->y = y;
-
-	if (GTK_WIDGET_CLASS (parent_class)->unmap != NULL) {
-		(* GTK_WIDGET_CLASS (parent_class)->unmap) (widget);
-	}
-}
-
-static void
-totem_playlist_realize (GtkWidget *widget)
-{
-	TotemPlaylist *playlist = TOTEM_PLAYLIST (widget);
-	int x, y;
-
-	g_return_if_fail (widget != NULL);
-
-	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
-
-	if (GTK_WIDGET_CLASS (parent_class)->realize != NULL) {
-		(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
-	}
-
-	x = gconf_client_get_int (playlist->_priv->gc,
-			GCONF_PREFIX"/playlist_x", NULL);
-	y = gconf_client_get_int (playlist->_priv->gc,
-			GCONF_PREFIX"/playlist_y", NULL);
-
-	if (x == -1 || y == -1
-			|| x > gdk_screen_width () || y > gdk_screen_height ())
-		return;
-
-	gtk_window_move (GTK_WINDOW (widget), x, y);
-}
-
 GtkWidget*
 totem_playlist_new (void)
 {
 	TotemPlaylist *playlist;
 	GtkWidget *container, *item;
-	char *filename;
 
 	playlist = TOTEM_PLAYLIST (g_object_new (GTK_TYPE_PLAYLIST, NULL));
 
@@ -1507,18 +1374,6 @@ totem_playlist_new (void)
 		return NULL;
 	}
 
-	gtk_dialog_set_has_separator (GTK_DIALOG (playlist), FALSE);
-	gtk_window_set_title (GTK_WINDOW (playlist), _("Playlist"));
-	gtk_dialog_add_buttons (GTK_DIALOG (playlist),
-			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-			NULL);
-	gtk_window_set_default_size (GTK_WINDOW (playlist),
-			300, 375);
-	g_signal_connect_object (G_OBJECT (playlist), "response",
-			G_CALLBACK (gtk_widget_hide), 
-			GTK_WIDGET (playlist),
-			0);
-
 	/* Connect the buttons */
 	item = glade_xml_get_widget (playlist->_priv->xml, "add_button");
 	g_signal_connect (G_OBJECT (item), "clicked",
@@ -1554,7 +1409,7 @@ totem_playlist_new (void)
 	container = glade_xml_get_widget (playlist->_priv->xml, "vbox4");
 	g_object_ref (container);
 	gtk_container_remove (GTK_CONTAINER (item), container);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (playlist)->vbox),
+	gtk_box_pack_start (GTK_BOX (playlist),
 			container,
 			TRUE,       /* expand */
 			TRUE,       /* fill */
@@ -1573,11 +1428,7 @@ totem_playlist_new (void)
 	playlist->_priv->icon =
 		totem_interface_load_pixbuf ("playlist-playing.png");
 
-	filename = totem_interface_get_full_path ("playlist-24.png");
-	gtk_window_set_icon_from_file (GTK_WINDOW (playlist), filename, NULL);
-	g_free (filename);
-
-	gtk_widget_show_all (GTK_DIALOG (playlist)->vbox);
+	gtk_widget_show_all (GTK_WIDGET (playlist));
 
 	return GTK_WIDGET (playlist);
 }
@@ -1625,6 +1476,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
 			NULL);
+	totem_playlist_update_save_button (playlist);
 
 	return TRUE;
 }
@@ -1638,6 +1490,7 @@ totem_playlist_add_mrl (TotemPlaylist *playlist, const char *mrl,
 	g_return_val_if_fail (mrl != NULL, FALSE);
 
 	res = totem_pl_parser_parse (playlist->_priv->parser, mrl, TRUE);
+
 	if (res == TOTEM_PL_PARSER_RESULT_UNHANDLED)
 		return totem_playlist_add_one_mrl (playlist, mrl, display_name);
 	if (res == TOTEM_PL_PARSER_RESULT_ERROR)
@@ -1645,6 +1498,8 @@ totem_playlist_add_mrl (TotemPlaylist *playlist, const char *mrl,
 		totem_playlist_error (_("Playlist error"), _("The playlist '%s' could not be parsed, it might be damaged."), playlist);
 		return FALSE;
 	}
+	if (res == TOTEM_PL_PARSER_RESULT_IGNORED)
+		return FALSE;
 
 	return TRUE;
 }
@@ -1662,6 +1517,8 @@ totem_playlist_clear (TotemPlaylist *playlist)
 	if (playlist->_priv->current != NULL)
 		gtk_tree_path_free (playlist->_priv->current);
 	playlist->_priv->current = NULL;
+	g_free (playlist->_priv->cdindex);
+	playlist->_priv->cdindex = NULL;
 }
 
 static void
@@ -1840,6 +1697,34 @@ char
 	return path;
 }
 
+gboolean
+totem_playlist_get_current_metadata (TotemPlaylist *playlist,
+				     char         **artist,
+				     char         **title,
+				     char         **album)
+{
+	GtkTreeIter iter;
+
+        g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
+
+	if (update_current_from_playlist (playlist) == FALSE)
+		return FALSE;
+
+	gtk_tree_model_get_iter (playlist->_priv->model,
+			&iter,
+			playlist->_priv->current);
+
+	*artist = NULL;
+	gtk_tree_model_get (playlist->_priv->model,
+			&iter,
+			CACHE_ARTIST_COL, artist,
+			CACHE_TITLE_COL, title,
+			CACHE_ALBUM_COL, album,
+			-1);
+
+	return (*artist != NULL);
+}
+
 gboolean
 totem_playlist_has_previous_mrl (TotemPlaylist *playlist)
 {
@@ -1898,7 +1783,7 @@ totem_playlist_has_next_mrl (TotemPlaylist *playlist)
 }
 
 gboolean
-totem_playlist_set_title (TotemPlaylist *playlist, const gchar *title)
+totem_playlist_set_title (TotemPlaylist *playlist, const char *title)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
@@ -1920,14 +1805,283 @@ totem_playlist_set_title (TotemPlaylist *playlist, const gchar *title)
 			FILENAME_COL, title,
 			-1);
 
+	g_signal_emit (playlist,
+		       totem_playlist_table_signals[ACTIVE_NAME_CHANGED], 0);
+
 	return TRUE;
 }
 
+#ifdef HAVE_LIBMUSICBRAINZ
+#define CDDATALEN 256
+
+typedef struct _TotemCDData {
+	gpointer data;
+	TotemPlaylist *list;
+} TotemCDData;
+
+typedef struct _TotemCDMetadata {
+	char **titles, **artists;
+	char *album, *artist;
+	unsigned int num_tracks;
+} TotemCDMetadata;
+
+static void
+totem_playlist_set_cd_metadata (TotemPlaylist *playlist, TotemCDMetadata *meta)
+{
+	GtkListStore *store;
+	GtkTreeIter iter, active_iter;
+	gchar *title;
+	unsigned int n;
+	gboolean res;
+	GValue v = { 0 };
+	const gchar *old_uri;
+
+	if (update_current_from_playlist (playlist) == FALSE)
+		return;
+
+	store = GTK_LIST_STORE (playlist->_priv->model);
+	gtk_tree_model_get_iter (playlist->_priv->model,
+				 &active_iter, playlist->_priv->current);
+	for (res = gtk_tree_model_get_iter_first (playlist->_priv->model,
+						  &iter); res != FALSE;
+	     res = gtk_tree_model_iter_next (playlist->_priv->model, &iter)) {
+		gtk_tree_model_get_value (playlist->_priv->model, &iter,
+					  URI_COL, &v);
+		old_uri = g_value_get_string (&v);
+		if (old_uri != NULL &&
+		    sscanf (old_uri, "cdda://%d", &n) == 1 &&
+		    n > 0 && n <= meta->num_tracks) {
+			n--;
+			title = g_strdup_printf ("%s - %s", meta->artists[n],
+						 meta->titles[n]);
+			gtk_list_store_set (store, &iter,
+					    FILENAME_COL, title,
+					    CACHE_ARTIST_COL, meta->artists[n],
+					    CACHE_TITLE_COL, meta->titles[n],
+					    CACHE_ALBUM_COL, meta->album,
+					    -1);
+			if (memcmp (&iter, &active_iter,
+				    sizeof (GtkTreeIter)) == 0) {
+				g_signal_emit (playlist,
+					       totem_playlist_table_signals[ACTIVE_NAME_CHANGED], 0);
+			}
+			g_free (title);
+		}
+		g_value_unset (&v);
+	}
+}
+
+static gboolean
+idle_set_metadata (TotemCDData * data)
+{
+	TotemCDMetadata *meta;
+	GList *item, *list = data->data;
+	TotemPlaylist *playlist = data->list;
+	unsigned int n;
+	char *text;
+
+	g_free (data);
+	if (!list)
+		return FALSE;
+
+	/* choose? */
+	if (g_list_length (list) > 1) {
+		GtkWidget *box, *dialog, *label, *menu;
+		gint res;
+
+		dialog = gtk_dialog_new_with_buttons (_("Select CD"), NULL, 0,
+				GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+		box = gtk_vbox_new (FALSE, 6);
+		gtk_container_set_border_width (GTK_CONTAINER (box), 6);
+
+		label = gtk_label_new (_("Please select the currently playing CD:"));
+		gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
+
+		menu = gtk_combo_box_new_text ();
+		for (n = 0; n  < g_list_length (list); n++) {
+			meta = g_list_nth_data (list, n);
+			if (meta->artist) {
+				text = g_strdup_printf ("%s - %s",
+							meta->artist,
+							meta->album);
+			} else {
+				text = g_strdup (meta->album);
+			}
+			gtk_combo_box_append_text (GTK_COMBO_BOX (menu), text);
+			g_free (text);
+		}
+		gtk_combo_box_set_active (GTK_COMBO_BOX (menu), 0);
+		gtk_box_pack_start (GTK_BOX (box), menu, TRUE, TRUE, 0);
+
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+				    box, TRUE, TRUE, 0);
+		gtk_widget_show_all (dialog);
+		res = gtk_dialog_run (GTK_DIALOG (dialog));
+		if (res == GTK_RESPONSE_OK) {
+			n = gtk_combo_box_get_active (GTK_COMBO_BOX (menu));
+			meta = g_list_nth_data (list, n);
+		} else {
+			meta = NULL;
+		}
+		gtk_widget_destroy (dialog);
+	} else {
+		meta = list->data;
+	}
+
+	/* put in playlist */
+	if (meta) {
+		totem_playlist_set_cd_metadata (playlist, meta);
+	}
+
+	/* clean up */
+	for (item = list; item != NULL; item = item->next) {
+		meta = item->data;
+		for (n = 0; n < meta->num_tracks; n++) {
+			g_free (meta->artists[n]);
+			g_free (meta->titles[n]);
+		}
+		g_free (meta->artist);
+		g_free (meta->album);
+		g_free (meta);
+	}
+	g_list_free (list);
+
+	return FALSE;
+}
+
+static gpointer
+cd_metadata_thread (TotemCDData *data)
+{
+	TotemPlaylist *playlist = data->list;
+	gchar *cdid = data->data;
+	musicbrainz_t mbo;
+	char *args[2] = { NULL, NULL }, str_data[CDDATALEN];
+	char artist[CDDATALEN], track[CDDATALEN];
+	gint i, n, t, x;
+	GList *metadata = NULL;
+	TotemCDMetadata *meta;
+	GConfClient *gc = gconf_client_get_default ();
+
+	g_free (data);
+	args[0] = cdid;
+
+	/* look up in musicbrainz */
+	mbo = mb_New ();
+	mb_UseUTF8 (mbo, 1);
+	mb_SetDepth (mbo, 2);
+	mb_SetMaxItems (mbo, 10);
+
+	/* proxy through GNOME */
+#define PROXY "/system/http_proxy/"
+	if (gconf_client_get_bool (gc, PROXY"use_http_proxy", NULL)) {
+		char *host;
+		int port;
+
+		host = gconf_client_get_string (gc, PROXY"host", NULL);
+		port = gconf_client_get_int (gc, PROXY"port", NULL);
+		if (host != NULL && port != 0) {
+			mb_SetProxy (mbo, host, port);
+		}
+		g_free (host);
+
+#if 0
+		/* proxy authentication, disabled because for some reason
+		 * the latest release of musicbrainz doesn't include this
+		 * yet (but their webdocs do). */
+		if (gconf_client_get_bool (gc, PROXY"use_authentication", NULL)) {
+			char *user, *pass;
+
+			user = gconf_client_get_string (gc,
+					PROXY"authentication_user", NULL);
+			pass = gconf_client_get_string (gc,
+					PROXY"authentication_password", NULL);
+			if (user && pass) {
+				mb_SetProxyCreds (mbo, user, pass);
+			}
+			g_free (user);
+			g_free (pass);
+		}
+#endif
+	}
+#undef PROXY
+
+	/* now get the artist info */
+	if (!mb_QueryWithArgs (mbo, MBQ_GetCDInfoFromCDIndexId, args))
+		goto fail;
+
+	n = mb_GetResultInt (mbo, MBE_GetNumAlbums);
+	for (i = 1; i <= n; i++) {
+		meta = g_new0 (TotemCDMetadata, 1);
+
+		mb_Select (mbo, MBS_Rewind);
+		mb_Select1 (mbo, MBS_SelectAlbum, i);
+		mb_GetResultData (mbo, MBE_AlbumGetAlbumName,
+				  str_data, CDDATALEN);
+		meta->album = g_strdup (str_data);
+		if (mb_GetResultData (mbo, MBE_AlbumGetArtistName,
+				      str_data, CDDATALEN))
+			meta->artist = g_strdup (str_data);
+		t = mb_GetResultInt (mbo, MBE_AlbumGetNumTracks);
+		meta->num_tracks = t;
+		meta->artists = g_new (gchar *, t);
+		meta->titles = g_new (gchar *, t);
+
+		for (x = 1; x <= t; x++) {
+			mb_GetResultData1 (mbo, MBE_AlbumGetArtistName,
+					   artist, CDDATALEN, x);
+			meta->artists[x-1] = g_strdup (artist);
+			mb_GetResultData1 (mbo, MBE_AlbumGetTrackName,
+					   track, CDDATALEN, x);
+			meta->titles[x-1] = g_strdup (track);
+		}
+
+		metadata = g_list_append (metadata, meta);
+	}
+
+	/* go back to main thread */
+	data = g_new (TotemCDData, 1);
+	data->list = playlist;
+	data->data = metadata;
+	g_idle_add ((GSourceFunc) idle_set_metadata, data);
+
+fail:
+	g_object_unref (G_OBJECT (gc));
+	mb_Delete (mbo);
+	g_free (cdid);
+
+	return NULL;
+}
+#endif
+
+void
+totem_playlist_set_cdindex (TotemPlaylist *playlist, const char *cdindex_id)
+{
+#ifdef HAVE_LIBMUSICBRAINZ
+	TotemCDData *data;
+
+	g_return_if_fail (cdindex_id != NULL);
+
+	/* caching */
+	if (playlist->_priv->cdindex &&
+	    strcmp (cdindex_id, playlist->_priv->cdindex) == 0)
+		return;
+	g_free (playlist->_priv->cdindex);
+	playlist->_priv->cdindex = g_strdup (cdindex_id);
+
+	data = g_new (TotemCDData, 1);
+	data->data = g_strdup (cdindex_id);
+	data->list = playlist;
+
+	g_thread_create ((GThreadFunc) cd_metadata_thread, data, FALSE, NULL);
+#endif
+}
+
 gboolean
 totem_playlist_set_playing (TotemPlaylist *playlist, gboolean state)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
+	GtkTreePath *path;
 
 	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), FALSE);
 
@@ -1949,6 +2103,16 @@ totem_playlist_set_playing (TotemPlaylist *playlist, gboolean state)
 		gtk_list_store_set (store, &iter,
 				PIX_COL, NULL,
 				-1);
+
+	if (state == FALSE)
+		return TRUE;
+
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
+	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (playlist->_priv->treeview),
+				      path, NULL,
+				      TRUE, 0.5, 0);
+	gtk_tree_path_free (path);
+	
 	return TRUE;
 }
 
@@ -2051,12 +2215,10 @@ totem_playlist_get_repeat (TotemPlaylist *playlist)
 void
 totem_playlist_set_repeat (TotemPlaylist *playlist, gboolean repeat)
 {
-	GtkWidget *button;
-
 	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
 
-	button = glade_xml_get_widget (playlist->_priv->xml, "repeat_button");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), repeat);
+	gconf_client_set_bool (playlist->_priv->gc, GCONF_PREFIX"/repeat",
+			repeat, NULL);
 }
 
 gboolean
@@ -2070,12 +2232,10 @@ totem_playlist_get_shuffle (TotemPlaylist *playlist)
 void
 totem_playlist_set_shuffle (TotemPlaylist *playlist, gboolean shuffle)
 {
-	GtkWidget *button;
-
 	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
 
-	button = glade_xml_get_widget (playlist->_priv->xml, "shuffle_button");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), shuffle);
+	gconf_client_set_bool (playlist->_priv->gc, GCONF_PREFIX"/shuffle",
+			shuffle, NULL);
 }
 
 void
@@ -2166,9 +2326,6 @@ totem_playlist_class_init (TotemPlaylistClass *klass)
 	parent_class = gtk_type_class (gtk_dialog_get_type ());
 
 	G_OBJECT_CLASS (klass)->finalize = totem_playlist_finalize;
-	GTK_WIDGET_CLASS (klass)->realize = totem_playlist_realize;
-	GTK_WIDGET_CLASS (klass)->unrealize = totem_playlist_unrealize;
-	GTK_WIDGET_CLASS (klass)->unmap = totem_playlist_unmap;
 
 	/* Signals */
 	totem_playlist_table_signals[CHANGED] =
@@ -2179,6 +2336,22 @@ totem_playlist_class_init (TotemPlaylistClass *klass)
 				NULL, NULL,
 				g_cclosure_marshal_VOID__VOID,
 				G_TYPE_NONE, 0);
+	totem_playlist_table_signals[ITEM_ACTIVATED] =
+		g_signal_new ("item-activated",
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (TotemPlaylistClass, item_activated),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
+	totem_playlist_table_signals[ACTIVE_NAME_CHANGED] =
+		g_signal_new ("active-name-changed",
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (TotemPlaylistClass, active_name_changed),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
 	totem_playlist_table_signals[CURRENT_REMOVED] =
 		g_signal_new ("current-removed",
 				G_TYPE_FROM_CLASS (klass),
