@@ -73,8 +73,11 @@ struct Totem {
 	BaconVideoWidget *bvw;
 	GtkWidget *prefs;
 	GtkWidget *statusbar;
+
+	/* Sidebar */
 	GtkWidget *sidebar;
 	gboolean sidebar_shown;
+	gboolean sidebar_ever_shown;
 
 	/* Separate Dialogs */
 	GtkWidget *properties;
@@ -85,6 +88,7 @@ struct Totem {
 	GtkWidget *pp_button;
 	/* fullscreen Play/Pause */
 	GtkWidget *fs_pp_button;
+	GtkTooltips *tooltip;
 
 	/* Seek */
 	GtkWidget *seek;
